class Agent {
  PVector position; // how should starting position be determined?
  PVector velocity; 
  PVector acceleration;
  PVector drag; 
  float size;  // "mass" of the particles
  color c;
  String letter;
  int trial_index;
  
  float[][] features;
  // each feature will be a parameter that drives phenotype of the agent
  // each sensor (containing 24 feature values) is ?????
  
  Agent(String letter, int trial_index) {
    features = new float[NUM_SENSORS][NUM_FEATURES];
    size = 5; // what should size be mapped to?? 
    this.letter = letter;
    this.trial_index = trial_index;
    
    c = color(map((int(letter.charAt(0)) - 65),0, 25, 0, 255),  // map hue to which letter it is
              map(trial_index, 1, 40, 50, 255), // map trial number to how saturated the color is
              255); // brightness value = max
  }
  
  void setupMotionParameters() {
    position = new PVector( map(features[0][0], -1, 1, width/2 - 100, width/2+100), 
                            map(features[1][0], -1, 1, height/2 - 100, height/2 + 100), 
                            map(features[2][0], -1, 1, depth/2 - 100, depth/2 + 100) );  
    // x = Mean Absolute Deviation (absolute mean over how much each sample value in the sample differs from mean), sensor 1
    // y = Mean Absolute Deviation, sensor 2
    // z = Mean Absolute Deviation, sensor 3
    
    size = map(features[3][0], -1, 1, 1, 20); // MAD of sensor 4 determines mass of particle
    
    
    // how fast the agent moves is determined by max - min of signal 
    // ("letters" with greater acceleration magnitudes travel faster)
    velocity = new PVector(features[0][1], features[1][1], features[2][1]);
    // x = Peak-To-Peak (signal absolute max - absolute min), sensor 1
    // y = Peak-To-Peak, sensor 2
    // z = Peak-To-Peak, sensor 3

    acceleration = new PVector( 0, 0, 0 );
    drag = new PVector( 0, 0, 0 );
  }
  
  void printGenotype() {
    println("Agent: " + letter + " Trial: " + trial_index); 
    for (int i = 0; i < NUM_SENSORS; i++) {
      println("sensor number " + (i+1));
      for (int j = 0; j < NUM_FEATURES; j++) {
        print(features[i][j] + " ");
      }
      println();
    }
    println("-----------------------------");
  }
  
  void display() {
    pushMatrix();
    translate(position.x, position.y, position.z);
    fill(c); 
    rotateX(cam.getRotations()[0]); //rotate towards the camera angle in X direction
    rotateY(cam.getRotations()[1]); //rotate towards the camera angle in Y direction
    rotateZ(cam.getRotations()[2]); //rotate towards the camera angle in Z direction
    ellipse(0, 0, 5, 5);
    popMatrix();
  }
  
  void calculateForces() {
    acceleration = new PVector( map(features[0][2], -1, 1, -maxAcc, maxAcc),
                                map(features[1][2], -1, 1, -maxAcc, maxAcc), 
                                map(features[2][2], -1, 1, -maxAcc, maxAcc) );
    // x = Spectral Centroid (center frequency value across signal after computing Discrete Fourier Transform), from sensor 1
    // y = Spectral Centroid, sensor 2
    // z = Spectral Centroid, sensor 3
    
    drag = new PVector( map(features[0][3], -1, 1, 0, 1), 
                        map(features[1][3], -1, 1, 0, 1), 
                        map(features[2][3], -1, 1, 0, 1) ); 
    // x = Frequency value at maximum magnitude of Discrete Fourier Transform, sensor 1
    // y = Frequency @ max, sensor 2
    // z = Frequency @ max, sensor 3
  }
  
  void move() {
    // TO DO: add drag into this
    if (velocity.mag() != 0) {
      position.x += velocity.x + (acceleration.x * size);
      position.y += velocity.y + (acceleration.y * size);
      position.z += velocity.z + (acceleration.z * size);
    
      acceleration.mult(0);
    }
  }
  
  void handleBoundaries() { // change direction if at the edge of the boundary
    if (position.x > width || position.x < 0) {
      velocity.x = 0; 
    }
    
    if (position.y > height || position.y < 0) {
      velocity.y = 0;  
    }
    
    if (position.z > depth || position.z < 0) {
      velocity.z = 0; 
    }
  }
  
}
